// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include "ELE491_Template.hpp"

// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------
#include "resources/lp.xpm"
#include "resources/hp.xpm"
//#include "wx/image.h"


// ----------------------------------------------------------------------------
// event tables and other macros for wxWindows
// ----------------------------------------------------------------------------

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(MENU_FILE_OPEN,  MyFrame::OnLoad)
    EVT_MENU(MENU_FILE_QUIT,  MyFrame::OnQuit)
    EVT_MENU(MENU_HELP_ABOUT, MyFrame::OnAbout)
    EVT_MENU(ID_ToGray, MyFrame::OnToGray)
    EVT_PAINT(MyFrame::OnPaint)
    EVT_MENU(MENU_FILTER_LP, MyFrame::OnFilter)
    EVT_MENU(MENU_FILTER_HP, MyFrame::OnFilter)
	
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyWindow, wxWindow)
    EVT_PAINT(MyWindow::OnPaint)
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// the application class
// ----------------------------------------------------------------------------

// 'Main program' equivalent: the program execution "starts" here
bool MyApp::OnInit()
{
    // create the main application window
    MyFrame *frame = new MyFrame(_T("ELE491_Template App"),
                                 wxPoint(50, 50), wxSize(800, 600));
                                 
    // and show it (the frames, unlike simple controls, are not shown when
    // created initially)
    frame->Show(TRUE);
    
    // Enable all image handlers - TIFF is among them
    ::wxInitAllImageHandlers(); 

    // success: wxApp::OnRun() will be called which will enter the main message
    // loop and the application will run. If we returned FALSE here, the
    // application would exit immediately.
    return TRUE;
}

// ----------------------------------------------------------------------------
// main frame
// ----------------------------------------------------------------------------

// frame constructor
MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, long style)
       : wxFrame(NULL, -1, title, pos, size, style)
{
    // set the frame icon
    SetIcon(wxICON(mondrian));

#if wxUSE_MENUS
    // create a menu bar
    wxMenu *fileMenu = new wxMenu;
    wxMenu *image_processMenu = new wxMenu;
        wxMenu *filterMenu    = new wxMenu;
    // the "About" item should be in the help menu
    wxMenu *helpMenu = new wxMenu;
    fileMenu->Append(MENU_FILE_OPEN, _T("&Open...\tCtrl-O"), _T("Open a new Image"));
    fileMenu->Append(MENU_FILE_QUIT, _T("E&xit\tAlt-X"), _T("Quit this program"));
    
    //build filterMenu
    filterMenu->Append(MENU_FILTER_LP, _T("&Lowpass Filter\tAlt-L"), _T("Lowpass Filter"));
    filterMenu->Append(MENU_FILTER_HP, _T("&Highpass Filter\tAlt-H"), _T("Highpass Filter"));
	

	
    image_processMenu->Append(MENU_FILTER,_T("Fil&ters\tAlt-T"),filterMenu, _T("Filter Menu"));
    
    helpMenu->Append(MENU_HELP_ABOUT, _T("&About...\tF1"), _T("Show about dialog"));


    // now append the freshly created menu to the menu bar...
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, _T("&File"));
    menuBar->Append(image_processMenu, _T("&Image Process"));
    menuBar->Append(helpMenu, _T("&Help"));

    // ... and attach this menu bar to the frame
    SetMenuBar(menuBar);
#endif // wxUSE_MENUS

#if wxUSE_TOOLBAR
    wxToolBar *MyToolBar = new wxToolBar(this, wxID_ANY);

    MyToolBar->AddTool(MENU_FILTER_LP,lp_xpm, _T("Lowpass Filter"));
    MyToolBar->AddTool(MENU_FILTER_HP,hp_xpm, _T("Highpass Filter"));
    
    MyToolBar->Realize();
    SetToolBar(MyToolBar);
     
#endif

#if wxUSE_STATUSBAR
    // create a status bar just for fun (by default with 1 pane only)
    CreateStatusBar(2);
    SetStatusText(_T("Welcome to ELE491!"));
#endif // wxUSE_STATUSBAR

    pImage = NULL;

    MySplitter = new wxSplitterWindow(this, wxID_ANY, wxPoint(150,150), wxSize(400,400), wxSP_3DSASH);
    rightWindow = new MyWindow(&rImage, MySplitter, wxID_ANY, wxDefaultPosition, wxSize(MySplitter->GetSize().GetWidth()-MySplitter->GetSashPosition(),MySplitter->GetSize().GetHeight()));
    leftWindow  = new MyWindow(&pImage, MySplitter, wxID_ANY, wxDefaultPosition, wxSize(MySplitter->GetSashPosition(),MySplitter->GetSize().GetHeight()));    
    rightWindow->Show(true);
    leftWindow->Show(true);
    MySplitter->SplitVertically(leftWindow, rightWindow);
}


// event handlers

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    // TRUE is to force the frame to close
    delete rightWindow;
    delete leftWindow;
    Close(TRUE);
}


void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    wxString msg;
    msg.Printf( _T("This is the About dialog for ELE491.\n")
                _T("Welcome to %s"), wxVERSION_STRING);

    wxMessageBox(msg, _T("About ELE491"), wxOK | wxICON_INFORMATION,this);
}
void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{   

    wxString fileName;
    wxString NameofFile;
    
    wxFileDialog fileDialog(NULL, "Choose a file to load ...", "", "", "*.tif", wxOPEN, wxDefaultPosition);
    if( fileDialog.ShowModal() == wxID_OK )
    
    {
        fileName = fileDialog.GetPath();
        NameofFile = fileDialog.GetFilename();
        //Update the view of main frame
        if( pImage != NULL ) delete pImage;
        pImage = new wxImage(fileName); 
        rImage = new wxImage(pImage->Copy());
                
        if( pImage == NULL )
            wxMessageBox("File Failed to Open!", _T("Error"), wxOK | wxICON_INFORMATION, this);        
        else
        {
            SetTitle(fileName);
            leftWindow->Refresh(); 
            rightWindow->Refresh();
 
            Refresh();
        }
    }
    fileDialog.Destroy();     
}
void MyFrame::OnPaint(wxPaintEvent& event)
{

    wxPaintDC dc(this);
    if( pImage == NULL )
    {
        event.Skip();
    }
    else
    {
        MySplitter->SetMinimumPaneSize(pImage->GetWidth());
        SetClientSize(pImage->GetWidth()*2+5,pImage->GetHeight());
    }
    
    return;    
}
void MyFrame::OnToGray(wxCommandEvent& event)
{
    if( pImage == NULL )
    {
        wxMessageBox("Image is not loaded yet!", _T("Error"), wxOK | wxICON_INFORMATION,this);
        return;
    }   
    
    int* buffer;
    buffer = new int[ pImage->GetWidth()*pImage->GetHeight() ];
    wxImage2grayBuffer(pImage,buffer);     
    
    wxImage* temp;
    temp = grayBuffer2wxImage(buffer, pImage->GetWidth(),pImage->GetHeight() );
    delete pImage;    
    pImage = temp;
    delete buffer;
    
    Refresh();
    return;    
}
void MyFrame::OnFilter( wxCommandEvent& event )
{
   
        
        if( pImage == NULL )
        {
            wxMessageBox("Image is not loaded yet!", _T("Error"), wxOK | wxICON_INFORMATION,this);
        } 
        else 
        {
            wxImage *Filtered = NULL;
            
            switch(event.GetId())
            {
               case MENU_FILTER_LP: Filtered = LowPass(rImage); break;
               case MENU_FILTER_HP: Filtered = HighPass(rImage); break;
            }  
            
            rImage = Filtered;
         }
    rightWindow->Refresh();
    return;
}

////////////////////////////////////////////////////////////////////////////////
// MyWindow Class Implementation
////////////////////////////////////////////////////////////////////////////////

//constructor
MyWindow::MyWindow(wxImage **image, wxWindow * parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxWindow(parent,id,pos,size,style,name)
{
    pwImage=image;
}

//OnPaint
void MyWindow::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);
    if( *pwImage == NULL )
    {
        event.Skip();

    }
    else
    {
        wxBitmap tempBitmap(**pwImage);//handler pointer to a pointer
        SetClientSize( tempBitmap.GetWidth(), tempBitmap.GetHeight() );
        dc.DrawBitmap(tempBitmap, 0,0, TRUE);
    }
    
    return;    
}
